#include <iostream>
#include <atomic>
#include <chrono>
#include <random>
#include "thread"

// Thread-safe counter class
class Counter {
private:
    int value;
public:
    Counter() : value(0) {}
    int next() {
        return ++value;
    }
};

// Thread-safe sum class
class SumAccumulator {
private:
    long total;
public:
    SumAccumulator() : total(0) {}
    void add(int value) {
        total += value;
    }
    long getTotal() const {
        return total;
    }
};

// Service function that reads the counter value, sleeps, and returns the value
int service(Counter& counter) {
    int value = counter.next();
    // Sleep for a random amount of time up to 1 second
    std::random_device rd;
    std::mt19937 gen(rd());
    std::uniform_int_distribution<> dis(1, 1000);
    std::this_thread::sleep_for(std::chrono::milliseconds(dis(gen)));
    return value;
}

int main() {
    Counter counter;
    SumAccumulator sumAccumulator;

    const int totalCalls = 100; // 1 million times

    auto startTime = std::chrono::high_resolution_clock::now();

    // Run the service synchronously
    for (int i = 0; i < totalCalls; ++i) {
        int value = service(counter);
        sumAccumulator.add(value);
    }

    auto endTime = std::chrono::high_resolution_clock::now();
    std::chrono::duration<double> duration = endTime - startTime;

    // Final total should be 500,000,500,000
    std::cout << "Final Total: " << sumAccumulator.getTotal() << std::endl;
    std::cout << "Execution Time: " << duration.count() << " seconds" << std::endl;

    return 0;
}
