#include <iostream>
#include <thread>
#include <atomic>
#include <vector>
#include <random>
#include <chrono>
#include <mutex>
#include <numeric>
#include <future>
#include "../../include/counter.h"
#include "../../include/sum.h"
#include "../../include/threadPool.h"

// Simulated service function
int service(Counter& counter) {
    int value = counter.getNext();

    // Sleep for a random amount of time up to 1 second
    std::random_device rd;
    std::mt19937 gen(rd());
    std::uniform_int_distribution<> dis(0, 1000);
    std::this_thread::sleep_for(std::chrono::milliseconds(dis(gen)));

    return value;
}

int main() {
    const int numCalls1 = 10000;
    const int maxThreads1 = 100;

    Counter counter;
    SumAccumulator sumAccumulator;

    auto worker = [&counter, &sumAccumulator]() {
        int value = service(counter);
        sumAccumulator.add(value);
    };

    ThreadPool pool(maxThreads1);

    std::cout << "Running service " << numCalls1 << " times with up to " << maxThreads1 << " threads..." << std::endl;

    auto startTime = std::chrono::high_resolution_clock::now();

    // Enqueue tasks for execution
    for (int i = 0; i < numCalls1; ++i) {
        pool.enqueue([&worker] {
            worker();
        });
    }

    // Wait for all tasks to complete
    pool.~ThreadPool();

    auto endTime = std::chrono::high_resolution_clock::now();
    std::chrono::duration<double> duration = endTime - startTime;

    std::cout << "Final Total: " << sumAccumulator.getTotal() << std::endl;
    std::cout << "Execution Time: " << duration.count() << " seconds" << std::endl;


    return 0;
}
